<?php

/**
 * This file contains \QUI\FormBuilder\Fields\Checkbox
 */

namespace QUI\FormBuilder\Fields;

use QUI;
use QUI\Exception;
use QUI\FormBuilder;
use QUI\Utils\Security\Orthos;

use function array_values;
use function implode;
use function is_array;

/**
 * Class Input
 *
 * @package QUI\FormBuilder\Fields
 */
class Checkbox extends FormBuilder\Field
{
    /**
     * @return string
     */
    public function getBody(): string
    {
        $result = '<div>';
        $data = $this->getAttribute('data');
        $choices = $this->getAttribute('choices');

        if (is_bool($data)) {
            $data = [];
        } else {
            if (!is_array($data)) {
                $data = [$data];
            }
        }

        $selected = [];

        foreach ($data as $entry) {
            if (is_array($entry)) {
                $entry = implode('-', $entry);
            }

            $parts = explode('-', $entry);
            $selectedEntry = array_pop($parts);

            $selected[] = (int)$selectedEntry;
        }

        foreach ($choices as $k => $choice) {
            $text = '';
            $checked = '';

            if (!empty($choice['text'])) {
                $text = $choice['text'];
            }

            if (isset($choice['checked']) && $choice['checked']) {
                $checked = 'checked="checked" ';
            }

            if (in_array($k, $selected)) {
                $checked = 'checked="checked" ';
            } elseif ($this->getParent() && $this->getParent()->isSend()) {
                $checked = '';
            }

            $error = '';

            if ($this->getAttribute('error')) {
                $error = ' class="qui-form-error"';
            }

            $choiceValue = $this->name . '-' . $k;

            $result .= '<label ' . $error . '>' .
                '<input type="checkbox"
                               name="' . $this->name . '[]"
                               value="' . $choiceValue . '" ' . $checked;

            if ($this->getAttribute('required')) {
                $result .= ' required';
            }

            $result .= '/>';
            $result .= '<span>' . $text . '</span>' .
                '</label>';
        }

        $result .= '</div>';

        return $result;
    }

    /**
     * Get text for the current value of the form field
     *
     * @return string
     */
    public function getValueText(): string
    {
        $value = '';
        $data = $this->getAttribute('data');

        if (is_array($data)) {
            $values = [];
            $choices = $this->getAttribute('choices');

            foreach ($data as $choice) {
                $choice = Orthos::clearFormRequest($choice);

                if (is_array($choice)) {
                    $choice = array_values($choice)[0];
                }

                $choice = explode('-', $choice);

                if (isset($choice[2])) {
                    $valueIndex = (int)$choice[2];

                    if (!empty($choices[$valueIndex]['text'])) {
                        $values[] = $choices[$valueIndex]['text'];
                    }
                }
            }

            $value = implode(', ', $values);
        }

        if (empty($value)) {
            $value = '-';
        }

        return $value;
    }

    /**
     * @return array
     */
    public function getCSSFiles(): array
    {
        return [
            URL_OPT_DIR . 'quiqqer/formbuilder/bin/fields/Checkbox.css'
        ];
    }

    /**
     * Check value for the input
     * @throws Exception
     */
    public function checkValue(): void
    {
        $data = $this->getAttribute('data');
        $required = $this->getAttribute('required');

        if ($required && empty($data)) {
            $this->setAttribute('error', true);

            throw new QUI\Exception([
                'quiqqer/formbuilder',
                'exception.missing.field'
            ]);
        }
    }
}
